// PLAYER CLASS EXTENDS PHASER.GAMEOBJECTS.SPRITE

import { GameOptions } from './gameOptions';

export class Player extends Phaser.GameObjects.Sprite {

    canMove : boolean;
    row : number;
    column : number;

    constructor(scene : Phaser.Scene) {
        super(scene, scene.game.config.width as number / 2, GameOptions.scrollY + 6, 'player');
        scene.add.existing(this);
        this.canMove = true;
        this.row = 0;
        this.column = 2;
    }

    move(goingLeft : boolean) : void {
        if (this.canMove) {
            if (goingLeft && (this.column > 0 || (this.row % 2 == 1))) {
                this.column -= (1 - this.row % 2);
                this.row ++;
                this.setFrame(0);
                this.lower(-1);
            }
            if (!goingLeft &&  this.column < GameOptions.gridSize.x - 1) {
                this.column += (this.row % 2);
                this.row ++;
                this.setFrame(1);
                this.lower(1);
            }
        }   
    }

    lower(delta : number) : void {
        let stepX : number = GameOptions.hexagon.width / 2 * delta;
        let stepY : number = GameOptions.hexagon.height / 4 * 3;
        this.canMove = false;
        let startPoint : Phaser.Math.Vector2 = new Phaser.Math.Vector2(this.x, this.y);
        let endPoint : Phaser.Math.Vector2 = new Phaser.Math.Vector2(this.x + stepX, this.y + stepY);
        let controlPoint1 : Phaser.Math.Vector2 = new Phaser.Math.Vector2(this.x + stepX, this.y + stepY / 2);
        let controlPoint2 : Phaser.Math.Vector2 = new Phaser.Math.Vector2(this.x + stepX, this.y + stepY / 2);
        let bezierCurve : Phaser.Curves.CubicBezier = new Phaser.Curves.CubicBezier(startPoint, controlPoint1, controlPoint2, endPoint);
        let tweenValue : any = {
            value : 0,
            previousValue : 0
        }
        this.scene.tweens.add({
            targets : tweenValue,
            value : 1,
            duration : 100,
            callbackScope : this,
            onComplete : () => {
                this.canMove = true;
            },
            onUpdate : (tween : Phaser.Tweens.Tween, target : any) => {
                let position : Phaser.Math.Vector2 = bezierCurve.getPoint(target.value);
                let prevPosition : Phaser.Math.Vector2 = bezierCurve.getPoint(target.previousValue);
                this.x += position.x - prevPosition.x;
                this.y += position.y - prevPosition.y;
                target.previousValue = target.value;
            }
        })
    }

    updatePosition() : void {
        if (this.y > GameOptions.scrollY) {
            let distance : number = (this.y - 6) / -25;
            this.y += distance;
            this.emit('toolow', distance);
        }
    }
}